/*
################################################################################

com.mediaworx.b4f.opencms.contenttypes/resources/spellcheckedtextinput/SpellcheckedTextInputWidget.js
mediaworx Berlin AG, Kai Widmann (widmann@mediaworx.com), 2013-03-22

FUNCTION:
	Integrates a spellchecked input box.
	Working in concert with the java class
	com.mediaworx.b4f.opencms.contenttypes.widgets.SpellcheckedTextInputWidget

INPUT PARAMETERS:
	configuration - Configuration string from the java class (see method getConfiguration)

################################################################################
*/

var spellcheckedTextInputWidgetDebug = false;

var spellcheckedTextInputWidgetNameCounter = {};

/**
 * generic method to create the widgets for the XML content editor and the inline editor
 *
 * @param configuration Configuration string from the java class (see method getConfiguration)
 * @param element       if inline==true, then the DOM-Element from the preview, null otherwise
 * @param inline        true if called from the inline editor, false otherwise
 *
 * @return Widget object
 */
function createGenericB4fSpellcheckedTextInputWidget(/*String*/configuration, /*Element*/element, /*boolean*/inline) {

	if (spellcheckedTextInputWidgetDebug) {
		console.log("createGenericB4fSpellcheckedTextInputWidget called. configuration: " + configuration);
	}

	// handle the configuration
	var b4fSpellcheckedTextInputConfig = {};
	var keyValuePairs = configuration.split("&");
	for (var i = 0; i < keyValuePairs.length; i++) {
		var keyValuePair = keyValuePairs[i].split("=");
		b4fSpellcheckedTextInputConfig[keyValuePair[0]] = keyValuePair[1];
	}

	// handle naming conflicts of repetitive elements
	if (spellcheckedTextInputWidgetNameCounter[b4fSpellcheckedTextInputConfig["name"]] >= 0) {
		spellcheckedTextInputWidgetNameCounter[b4fSpellcheckedTextInputConfig["name"]]++;
		b4fSpellcheckedTextInputConfig["name"] = b4fSpellcheckedTextInputConfig["name"] + "_" + spellcheckedTextInputWidgetNameCounter[b4fSpellcheckedTextInputConfig["name"]];
	}
	spellcheckedTextInputWidgetNameCounter[b4fSpellcheckedTextInputConfig["name"]] = 0;
	if (spellcheckedTextInputWidgetDebug) {
		console.log("SpellcheckedTextInputWidget - configured name: " + b4fSpellcheckedTextInputConfig["name"]);
	}

	/*
	 * The widget object definition
	 */
	var widget = {
		_value: "",
		_element: element,
		_input: null,
		_active: true,
		_isFocused: false,

		/**
		 * Returns the widget root element.
		 */
		getElement: function () {
			if (spellcheckedTextInputWidgetDebug) {
				console.log("SpellcheckedTextInputWidget.getElement called for " + b4fSpellcheckedTextInputConfig["name"]);
			}

			if (!inline) {
				this._element = document.createElement("div");
				this._element.setAttribute("class", "spellcheckedinput");

				this._input = document.createElement("input");
                this._input.type = "text";
                this._input.name = b4fSpellcheckedTextInputConfig["name"];
				this._input.setAttribute("spellcheck", "true");
				this._input.onkeyup = this.onTextChange;
				this._input.onblur = this.onTextChange;
				this._input.onkeypress = this.handleKeyPress;

				this._element.appendChild(this._input);
			}
			else {
				this._element.name = b4fSpellcheckedTextInputConfig["name"];
				this._element.setAttribute("contentEditable", "true");
				// Don't allow enter in Inline Text Inputs
				this._element.onkeydown = this.filterEnter;
				this._element.onkeypress = this.handleKeyPress;
				this._element.onkeyup = this.onTextChange;
				this._element.onblur = this.onTextChange;
				this._element.onpaste = this.onPaste;
				this.updateValue(this._element.innerHTML, false);
			}

			this._isFocused = false;

			// get the focusedNode
			var parent = window.getSelection().focusNode;

			// walk up the dom tree to find out if this div is the one that was clicked on
			// to avoid walking up the whole dom tree this is limited to 2 ancestor levels
			for (var i = 0; i < 2; i++) {
				if (parent == this._element) {
					this._isFocused = true;
					break;
				}
				else {
					try {
						parent = parent.parentNode;
					}
					catch (e) {
						break;
					}
				}
			}

			if (spellcheckedTextInputWidgetDebug) {
				console.log(this._element.name + (this._isFocused ? " is focused" : " is not focused"));
			}

			if (this._isFocused) {
				if (spellcheckedTextInputWidgetDebug) {
					console.log("forcing spellcheck for focused textinput now");
				}
				widget._element.blur();
				widget._element.focus();
			}

			return this._element;
		},

		/**
		 * filters the Enter key on key press events, because inserting a line break is not allowed
		 * @param event the key press event
		 * @returns {boolean}   false, if enter was pressed, true otherwise
		 */
		filterEnter: function(event) {
			// if Enter was pressed ...
			if (event.keyCode == 13) {
				// ... don't allow that key press
				event.stopPropagation();
				return false;
			}
			return true;
		},

		/**
		 * handles key press events and checks if there's a max length for the input field
		 * @param event the key press
		 * @returns true, if the key press is allowed, false if there's a max length that has been reached
		 */
		handleKeyPress: function(event) {
			if (b4fSpellcheckedTextInputConfig["maxlength"]) {
				if (
						b4fSpellcheckedTextInputConfig["maxlength"]
						&& event.which
						&& event.which >= 32
						&& !event.ctrlKey
				) {
					var maxlength = b4fSpellcheckedTextInputConfig["maxlength"];
					var value = inline ? widget._element.innerHTML : widget._input.value;
					value = widget.cleanupString(value);

					var selection = window.getSelection();
					var selectionLength = 0;

					// only handle cases with exactly one selection range
					if (selection.rangeCount == 1) {
						var range = selection.getRangeAt(0);

						// only handle selections within the editable div's content
						if (range.startContainer == widget._element.firstChild && range.startContainer == range.endContainer) {
							selectionLength = range.endOffset - range.startOffset;
						}
					}

					if ((value.length - selectionLength) >= maxlength) {
						event.stopPropagation();
						return false;
					}
				}
			}
			return true;
		},

		/**
		 * handles text change events, updates the widget value and checks if there's a max length for the input field
		 */
		onTextChange: function () {
			var value = inline ? widget._element.innerHTML : widget._input.value;
			value = widget.cleanupString(value);

			if (b4fSpellcheckedTextInputConfig["maxlength"] && value.length > b4fSpellcheckedTextInputConfig["maxlength"]) {
				var maxlength = b4fSpellcheckedTextInputConfig["maxlength"];
				value = value.substr(0, maxlength);
				if (inline) {
					widget._element.innerHTML = value;
					var range = document.createRange();
					range.selectNodeContents(widget._element);
					range.collapse(false);
					var selection = window.getSelection();
					selection.removeAllRanges();
					selection.addRange(range);
				}
				else {
					widget._input.value = value;
					widget._input.setSelectionRange(maxlength, maxlength);
				}
				alert("In diesem Feld sind maximal " + maxlength + " Zeichen erlaubt");
			}
			widget.updateValue(value, true);
		},

		/**
		 * Called whenever the user pastes a String to the widget. Strips html tags and entities from the pasted String
		 * and replaces occurrences of consecutive whitespace with one space.
		 * @returns {boolean} always returns <code>true</code>
		 */
		onPaste: function () {
			// determine the selection
			var originalContentLength = widget.cleanupString(widget._element.innerHTML).length;
			var selection = window.getSelection();
			var range;
			var selectionStartIndex = 0;
			var selectionEndIndex = 0;

			// only handle cases with exactly one selection range
			if (selection.rangeCount == 1) {
				range = selection.getRangeAt(0);

				// only handle selections within the editable div's content
				if (range.startContainer == widget._element.firstChild && range.startContainer == range.endContainer) {
					selectionStartIndex = range.startOffset;
					selectionEndIndex = range.endOffset;
				}
			}

			// handle the pasted content after a short delay (onPaste is called right before the string is pasted)
			window.setTimeout(
					function() {
						// get, cleanup and set the content
						var content = widget._element.innerHTML;
						content = widget.cleanupString(content);
						widget._element.innerHTML = content;

						// Place the cursor right after the inserted text
						var lengthAfterPaste = content.length;
						var lengthSelection = selectionEndIndex - selectionStartIndex;
						var pastedContentLength = lengthAfterPaste - (originalContentLength - lengthSelection);
						var cursorIndex = selectionStartIndex + pastedContentLength;

						var newSelectionRange = document.createRange();
						newSelectionRange.setStart(widget._element.firstChild, cursorIndex);
						newSelectionRange.setEnd(widget._element.firstChild, cursorIndex);
						selection.removeAllRanges();
						selection.addRange(newSelectionRange);
					}, 1);
			return true;
		},

		/**
		 * Will be called once the widget element is attached to the DOM.
		 */
		onAttachWidget: function () {
			if (spellcheckedTextInputWidgetDebug) {
				console.log("SpellcheckedTextInputWidget.onAttachWidget called for " + b4fSpellcheckedTextInputConfig["name"]);
			}
		},

		/**
		 * Returns if the widget is active.
		 */
		isActive: function () {
			if (spellcheckedTextInputWidgetDebug) {
				console.log("SpellcheckedTextInputWidget.isActive called for " + b4fSpellcheckedTextInputConfig["name"]);
			}
			return this._active;
		},

		/**
		 * Activates or deactivates the widget.
		 */
		setActive: function (/*boolean*/active) {
			if (spellcheckedTextInputWidgetDebug) {
				console.log("SpellcheckedTextInputWidget.setActive called for " + b4fSpellcheckedTextInputConfig["name"]);
			}
			this._active = active;
		},

		/**
		 * Returns the widget value.
		 */
		getValue: function () {
			return this.cleanupString(this._value);
		},


		/**
		 * updates the widget value and filters out any newlines ("<br>" or "<br/>") if the inline editor is used
		 * @param value     the new value
		 * @param fireEvent if true, the onChangeCommand is fired
		 */
		updateValue: function (/*String*/value, /*boolean*/fireEvent) {
			this._value = value;
			if (fireEvent && typeof this.onChangeCommand === "function") {
				this.onChangeCommand();
			}
		},


		/**
		 * Sets the widget value and fires the change event if required.
		 * @param value     the new value
		 * @param fireEvent if true, the onChangeCommand is fired
		 */
		setValue: function (/*String*/value, /*boolean*/fireEvent) {
			if (spellcheckedTextInputWidgetDebug) {
				console.log("SpellcheckedTextInputWidget.setValue called for " + b4fSpellcheckedTextInputConfig["name"] + ". value: " + value);
			}
			value = this.cleanupString(value);
			if (!inline) {
				this._input.value = value;
			}
			this.updateValue(value, fireEvent);
		},

		/**
		 * Strips html tags and entities from the given String and replaces occurences of consecutive whitespace with
		 * one space.
		 * @param input the String to be cleaned
		 * @returns {String} the cleaned String
		 */
		cleanupString: function  (/*String*/ input) {
			input = htmlEntityDecode(stripTags(input));
			return input;
		},

		/**
		 * Delegates the value change event to the editor.
		 * This function will be attached to the widget by the editor.
		 *
		 * It is required to call this function everytime the widget value changes.
		 * Changes that are propagated through this function won't be saved.
		 */
		onChangeCommand: null,

		/**
		 * Delegates the focus event to the editor.
		 * This function will be attached to the widget by the editor.
		 *
		 * It is required to call this function on widget focus, so the editor
		 * highlighting can be updated.
		 */
		onFocusCommand: null
	};
	return widget;
}

/**
 * Initialization function, creates a new SpellcheckedTextInput-Widget. The name of this function must be the one
 * returned by the method "getInitCall" in the backing Java class "SpellcheckedTextInputWidget"
 */
function b4fSpellcheckedTextInputWidgetInitializationCall() {

	registerWidgetFactory({

		/**
		 * The widget name and identifier.
		 */
		widgetName: "SpellcheckedTextInputWidget",

		/**
		 * Creates a new widget instance according to the configuration.
		 */
		createNativeWidget: function (/*String*/configuration) {
			return createGenericB4fSpellcheckedTextInputWidget(configuration, null, false);
		},


		/**
		 * Creates a new inline editing widget instance according to the configuration.
		 */
		createNativeWrapedElement: function (/*String*/configuration, /*Element*/ element) {
			return createGenericB4fSpellcheckedTextInputWidget(configuration, element, true);
		}
	});
}
